#pragma rtGlobals=1		// Use modern global access method.

///////////////////////////////////////Saving///////////////////////////////

//Makes an attribute wave
Function MakeAttribWaveInFolder(Folder, ResultName)
	String Folder, ResultName

	String TestPath
	DFREF FullPath
	
	TestPath="root:"+Folder
	
	If(DataFolderExists(Folder))
		FullPath = $Folder
	ElseIf(DataFolderExists(TestPath))
		FullPath = $TestPath
	Else
		String AlertMessage = "The specified path, <"+Folder+">, does not exist."
		Doalert 0, AlertMessage
		Return -1
	Endif
	
	DFREF saveDFR = GetDataFolderDFR()		//save current data folder
	SetDataFolder FullPath

	Variable NumWv
	NumWv = CountObjectsDFR(FullPath,1)
	
	Make /T/o/free/n=(NumWv,2) T_WavesInFolder = ""
	SetDimLabel 1,0,WaveName, T_WavesInFolder
	SetDimLabel 1,1,Type, T_WavesInFolder
	
	
	T_WavesInFolder[][0]=GetIndexedObjNameDFR(FullPath,1,p)
	
	Note /k	T_WavesInFolder, "EDB_Attributes"
	Duplicate/o T_WavesInFolder, $ResultName

	SetDataFolder saveDFR		//restore initial data folder
	return 1
End


//////////////////////////////
//Checks if waves from aatribwave are missing in folder
Static Function MissingAttribWave(AttribWave)
//Checks whether all the names in AttribWave actually exist as waves in its folder
	Wave/t AttribWave

	DFREF Folder = GetWavesDataFolderDFR(AttribWave)
	DFREF saveDFR = GetDataFolderDFR()		//save current data folder
	SetDataFolder Folder
	
	Variable NumWv = DimSize(AttribWave,0), ii, result = -1

	For(ii=0;ii<NumWv;ii+=1)
	
		if (Exists(AttribWave[ii][0]) != 1)
			result = ii
			break
		EndIf
	EndFor
	
	SetDataFolder saveDFR		//restore initial data folder
	Return Result
End
//////////////////////////////

//saves waves,specified by attribwave, in folder of attribwave with their respective attributes in a hdf5 file
Function HDF5SaveWithAttrib(AttribWave,AttribType,PathName,fileNameStr,[overwrite,GroupName])
	Wave/t AttribWave
	String PathName,fileNameStr, AttribType,GroupName
	variable overwrite

	DFREF Folder = GetWavesDataFolderDFR(AttribWave)
	String FolderStr = GetWavesDataFolder(AttribWave,0)
	DFREF saveDFR = GetDataFolderDFR()		//save current data folder
	String WvName
	variable ov, result=1
	
	if(paramisdefault(overwrite))
		ov=0
	else
		ov=overwrite
	endif	
	
	if(paramisdefault(GroupName))
		GroupName=FolderStr
	endif
	
	SetDataFolder Folder
	
	Variable NumWv = DimSize(AttribWave,0), ii, fileID, GroupID
	
	Variable Missing = MissingAttribWave(AttribWave)
	If(Missing >-1)
		String AlertString ="<"+AttribWave[Missing][0] + "> is not in the folder."
		DoAlert 0, AlertString
		return -1
	EndIf
	
	NewPath /o/q SWA_path, pathName
	
	Make /o/T/free/n=(1)  DB_Type 
	
	HDF5OpenFile /p=SWA_Path fileID as fileNameStr
	if(v_flag!=0)
		return -2
	endif
	
	HDF5CreateGroup  FileID,GroupName,GroupID
	
	For(ii=0;ii<NumWv;ii+=1)
		WvName = AttribWave[ii][0]
		DB_Type = {AttribWave[ii][1]}
		
		if(!StringMatch(DB_Type[0],"IGNORE"))
		
			Wave ToBeSaved = $WvName
		
			if(ov)
				HDF5SaveData /O ToBeSaved, GroupID
				HDF5SaveData /A=AttribType DB_Type,GroupID,WvName
			else
				HDF5SaveData /z ToBeSaved, GroupID
				if(v_flag==0)
					HDF5SaveData /A=AttribType DB_Type,GroupID,WvName
				else
					result = -3
				endif
			endif
		
		endif
	
	EndFor
	
	HDF5CloseFile fileID
	SetDataFolder saveDFR		//restore initial data folder
	return result
End

//////////////////////////////
//gets attributes from a text file 
Function GetAttributesFromFile(PathName,FileNameStr)
	String PathName, FileNameStr
	
	NewPath /o/q GA_Path PathName
	
	Variable NumWaves = DimSize(AttribWave,0), ii, jj
	DFREF AtrribWaveFolder = GetWavesDataFolderDFR(AttribWave)
	If(DimSize(AttribWave,1)<2)
		DoAlert 0, NameOfWave(AttribWave)+" doesn't have the right number of dimensions."
		Return -1
	Endif
	
	Variable FileRefNum, len, LineNum = 0, PointPos
	String buffer, ctl
	
	Open /r/p=GA_Path FileRefNum as FileNameStr
	if(v_flag !=0)
		return -1
	endif
	
	Make /o/t/n=1 W_FromFile=""
	
	do
		FReadLine FileRefNum, buffer
		len = strlen(buffer)
		if (len == 0)
			break						// No more lines to be read
		endif
		ctl=buffer[0,1]
		buffer = ReplaceString("\r",buffer,"")	//remove carriage returns
		if(stringmatch(ctl,"//")==0)	//ignore commented lines
			W_FromFile[LineNum]={buffer}
			LineNum+=1
		Endif
				
	while(1)
	
	Redimension /n=(-1,2) W_FromFile
	
	For(ii=0;ii<LineNum;ii+=1)	//divide left and right sides of equation into first and second column
		buffer = W_FromFile[ii][0]
		len = strlen(buffer)
		PointPos = strsearch(buffer,"=",0)
		W_FromFile[ii][0] = buffer[0,pointpos-1]
		W_FromFile[ii][1] = buffer[pointpos+1,len-1]
	EndFor
	
	Close FileRefNum
	Return LineNum	
End

////////////////////

Function SetAttributesFromFile(AttribWave, W_FromFile)
	Wave /t  AttribWave, W_FromFile
	
	Variable NumTypes = Dimsize(W_FromFile,0), WildCard, MatchLen, CompLen, ii, jj
	Variable NumWaves = DimSize(AttribWave,0)
	String MatchStr, CompStr, buffer
	
	
	For(jj=0;jj<NumTypes;jj+=1)
		buffer=W_FromFile[jj][0]
		
		WildCard=grepstring(buffer[0],"\*")	// * is wildcard character in Igor --> use \* to get '*'
		if(WildCard)	//wildcard
			matchstr=buffer[1,strlen(buffer)-1]		
		else //exact match
			matchstr=buffer
		endif
		MatchLen=StrLen(matchStr)
		
	
		For(ii=0;ii<NumWaves;ii+=1)
		CompStr = AttribWave[ii][0]
			if(WildCard)	//wildcard				
				CompLen=StrLen(CompStr)
				CompStr = CompStr[CompLen-MatchLen,CompLen-1]	//get suffix only
			endif
						
			if (stringMatch(CompStr,MatchStr))
					AttribWave[ii][1]=W_FromFile[jj][1]					
			endif
			
		EndFor
		
	EndFor
	
End

///////////////////////////////////////Loading///////////////////////////////
//Loads Attributes from a hdf5 file

Function HDF5GetAttrib(PathName,fileNameStr)
	String PathName,fileNameStr
	
	Variable FileID, nGroups, ii, nItems, jj
	String HDF5Groups
	NewPath /o/q GA_path, pathName
	
	HDF5OpenFile /r/p=GA_Path fileID as fileNameStr
	
	HDF5ListGroup /TYPE=1 /F/R, FileID, "/"
	
	nGroups = ItemsInList(S_HDF5ListGroup)
	
	Make /o/t/n=(nGroups,3) GA_GroupList
	
	
	
	GA_GroupList[][0]=StringFromList(p,S_HDF5ListGroup)	//Folders in root
	
	For(ii=0;ii<nGroups;ii+=1)
				
		HDF5ListGroup /TYPE=2 /F/R, FileID,GA_GroupList[ii][0]
		GA_GroupList[ii][1]=S_HDF5ListGroup					//Data in Folders
		nItems = ItemsInList(S_HDF5ListGroup)
		
		For(jj=0;jj<nItems;jj+=1)
		
			HDF5ListAttributes FileID, StringFromList(jj, S_HDF5ListGroup)
		
			GA_GroupList[ii][2+jj]={S_HDF5ListAttributes}				//Attributes of Data
					
		EndFor
	EndFor
	
	//KillWaves/z GA_GroupList
	HDF5CloseFile fileID
End

////////////////////////////////////////////////////////////


Function HDF5LoadAttrib(PathName,fileNameStr,Attrib, GroupList)
	String PathName,fileNameStr,Attrib
	Wave /t GroupList
	
	Variable nFolders = dimsize(GroupList,0), maxItems = dimsize(GroupList,1), nItems
	Variable ii,jj, FileID, GroupID, ItemNr, WLI, counter=0, Type
	String Item, TheItem, ItemList, AttList
	
	if(nFolders==0)
		nFolders=1
	endif
	
	NewPath /o/q GA_path, pathName
	HDF5OpenFile /r/p=GA_Path fileID as fileNameStr
	make /t/n=(2,1)/o GA_Attributes
	
	Make/Free/n=(nFolders) ItemsInFolder
	
	For(ii=0;ii<nFolders;ii+=1)		//Folders
		ItemList=GroupList[ii][1]
		nItems=ItemsInList(ItemList)
		For(jj=0;jj<nItems;jj+=1)
			TheItem=StringFromList(jj,ItemList)
			AttList=GroupList[ii][jj+2]
			WLI=WhichListItem(Attrib,AttList,";",0,0)
			
			if(WLI>-1)
	
				HDF5LoadData /z/Q/A=Attrib/IGOR=-1 /n=w_AttWv/o FileID, TheItem
				if(v_flag==0)
					Type=WaveType($"w_AttWv",1)
				else
					print "Error loading "+TheItem
				endif
				
				if(Type==2)
					wave/z/t w_attwv
					
					GA_Attributes[counter][0]={TheItem}
					GA_Attributes[counter][1]={w_AttWv}
					counter+=1
				else
					GA_Attributes[counter][0]={TheItem}
					GA_Attributes[counter][1]={"Error:Non-String Attribute"}
					counter+=1
				endif
				
			
			EndIf
	
		EndFor
	EndFor
	
		KillWaves /z AttWv
	HDF5CloseFile fileID
	return counter-1
End


////////////////////////////////////////////////////////////

Function/t HDF5LoadDataByAttrib(PathName,fileNameStr,WhichAttrib, AttribList)
	String PathName,fileNameStr,WhichAttrib
	Wave /t AttribList
	
	Variable nItems=Dimsize(AttribList,0), ii, FileID
	String Item, Attrib, LoadedWaves=""
	
	NewPath /o/q GA_path, pathName
	HDF5OpenFile /r/p=GA_Path fileID as fileNameStr
	
	For(ii=0;ii<nItems;ii+=1)
		Item=AttribList[ii][0]
		Attrib=AttribList[ii][1]

		if(stringMatch(Attrib,WhichAttrib))
	
			HDF5LoadData /IGOR=-1/Q FileID, Item
			LoadedWaves=LoadedWaves+s_WaveNames
		
		EndIf		
		
	EndFor
	
	HDF5CloseFile fileID
	return LoadedWaves
End

////////////////////////////////////////////////////////////

Function/t MakeAttribListStr(AttribList)
	Wave/t AttribList
	
	Variable nItems=Dimsize(AttribList,0), ii
	String AttribListStr=""
	
	For(ii=0;ii<nItems;ii+=1)
	
		if(WhichListItem(AttribList[ii][1],AttribListStr)==-1 && StrLen(AttribList[ii][1]))
	
			AttribListStr+=AttribList[ii][1]+";"
		
		Endif
	
	EndFor
	AttribListStr=SortList(AttribListStr,";",16)
	return AttribListStr
End