#pragma rtGlobals=1		// Use modern global access method.

Menu "Macros"	
	"Load ScanImage .tif", LoadScanImage()
	"Read Header", ReadHeader()
	"Read Header - Safe", ReadHeader(StrWaveName="HeadWave")


End


Function ReadHeader([StrWaveName])		//Reads Header of a ScanImage Tif file
String StrWaveName

variable FileRefNum, FilePos = 0, Condition = 1, counter = 0
String Line

Open /R /M="Select a ScanImage .tif file." /T=".tif" FileRefNum as "ImageFile"
FStatus FileRefNum

if (!(V_Flag))		//Abort if cancelled
	return -1
endif

If (paramisdefault(StrWaveName))
	Variable PointPos
	PointPos = strsearch(S_Filename, ".tif", 0)
	StrWaveName = S_FileName[0,PointPos-1]+"_header"
endif	



Do			//Find start of information

fSetPos FileRefNum, FilePos
fReadLine /T=(num2char(13)) FileRefNum, Line
	
	If (FilePos >= V_logEOF)
		DoAlert 0, "No occurence of a ScanImage header found in file <"+S_FileName+">."
		Return -1
	Elseif (stringmatch(Line[0,4], "state"))
		Condition = 0
	Else
		Condition = 1
	Endif
	

FilePos +=1
While (Condition)

Make /t/o/n=(150) Stringwave

filepos -=1
Condition = 1
Do			//Read header info

	fSetPos FileRefNum, FilePos
	fReadLine /T=(num2char(13)) FileRefNum, Line
	
	Stringwave[counter] = Line

	If (FilePos >= V_logEOF)
		Condition = 0
	Elseif (stringmatch(Line[0,4], "state") != 1)
		Condition = 0
	Elseif (counter > 150)
		DoAlert 0, "Found more Lines than StringWave has points. Readjust Function!"
		Condition = 0
	Endif

Counter +=1
FilePos += strlen(Line)
While (Condition)




Close FileRefNum

Redimension /n=(counter-1) StringWave
Duplicate /t/o Stringwave, $StrWaveName
Killwaves /z StringWave


Return Counter -1
End

/////////////////////////////////////////////////////////

Function /t LoadScanImage()

String StrWaveName, ImgWaveName

variable FileRefNum, FilePos = 0, Condition = 1, counter = 0
String Line

Open /R /M="Select a ScanImage .tif file." /T=".tif" FileRefNum as "ImageFile"
FStatus FileRefNum


if (!(V_Flag))		//Abort if cancelled
	return "-1"
endif

	Variable PointPos
	PointPos = strsearch(S_Filename, ".tif", 0)
	ImgWaveName = S_FileName[0,PointPos-1]
	ImgWaveName = ReplaceString("-", ImgWaveName, "_")
	StrWaveName = S_FileName[0,PointPos-1]+"_header"
	StrWaveName = ReplaceString("-", StrWaveName, "_")



Do			//Find start of information

fSetPos FileRefNum, FilePos
fReadLine /T=(num2char(13)) FileRefNum, Line
	
	If (FilePos >= V_logEOF)
		DoAlert 0, "No occurence of a ScanImage header found in file <"+S_FileName+">."
		Return "-1"
	Elseif (stringmatch(Line[0,4], "state"))
		Condition = 0
	Else
		Condition = 1
	Endif
	

FilePos +=1
While (Condition)

Make /t/o/n=(150) Stringwave

filepos -=1
Condition = 1
Do			//Read header info

	fSetPos FileRefNum, FilePos
	fReadLine /T=(num2char(13)) FileRefNum, Line
	
	Stringwave[counter] = Line

	If (FilePos >= V_logEOF)
		Condition = 0
	Elseif (stringmatch(Line[0,4], "state") != 1)
		Condition = 0
	Elseif (counter > 150)
		DoAlert 0, "Found more Lines than StringWave has points. Readjust Function!"
		Condition = 0
	Endif

Counter +=1
FilePos += strlen(Line)
While (Condition)

variable zdim = FramesFromHeader(StringWave), xdim, ydim, timeperframe, zoom
string loadfilename = S_path+s_filename

ImageLoad /q/T=tiff/C=-1/N=$imgWaveName loadfilename

xdim = dimsize($imgWaveName, 0)
ydim = dimsize($imgWaveName, 1)
timeperframe = ydim * sPerLineFromHeader(StringWave)
zoom= ZoomFromHeader(StringWave)

setscale /p x,0, (665.6/zoom/xdim), "m", $imgWaveName
setscale /p y,0, (665.6/zoom/ydim), "m", $imgWaveName
setscale /p t,0, (timeperframe), "s", $imgWaveName

note $imgWaveName, "Date: "+ExpDateFromHeader(StringWave)
note $imgWaveName, "Zoomfactor="+num2str(zoom)
note $imgWaveName, "ms/line="+num2str(msPerLineFromHeader(StringWave))
note $imgWaveName, "Number of frames="+num2str(dimsize($imgWaveName, 2))
note $imgWaveName, "Number of slices="+num2str(ZSlicesFromHeader(StringWave))
note $imgWaveName, "Z stepsize (no units!)="+num2str(ZStepsizeFromHeader(StringWave))

//665.6 m per side @ zoom1

Close FileRefNum

Redimension /n=(counter-1) StringWave
Duplicate /t/o Stringwave, $StrWaveName
Killwaves /z StringWave

return ImgWaveName
end

//////////////////////////////////////////////////////

Function ZoomFromHeader(HeaderWave)
wave /t HeaderWave
variable ZoomFactor

ZoomFactor = NumberByKey("state.acq.zoomones", HeaderWave[15], "=")
ZoomFactor += 10 * NumberByKey("state.acq.zoomtens", HeaderWave[14], "=")
ZoomFactor += 10 * NumberByKey("state.acq.zoomhundreds", HeaderWave[13], "=")

//ZoomFactor *= NumberByKey("state.acq.zoomFactor", HeaderWave[12], "=")	//? check for errors

Return ZoomFactor
End

//////////////////////////////////////////////////////

Function FramesFromHeader(HeaderWave)
wave /t HeaderWave
variable Frames

Frames = NumberByKey("state.acq.numberOfFrames", Headerwave[6], "=")

return Frames
End

//////////////////////////////////////////////////////

Function msPerLineFromHeader(HeaderWave)
wave /t HeaderWave

Return 1000*NumberByKey("state.acq.msPerLine", Headerwave[28], "=")

End

//////////////////////////////////////////////////////

Function sPerLineFromHeader(HeaderWave)
wave /t HeaderWave

Return NumberByKey("state.acq.msPerLine", Headerwave[28], "=")

End

//////////////////////////////////////////////////////

Function /t ExpDateFromHeader(HeaderWave)
wave /t HeaderWave
string ExpDate, month, day, year, ExpTime
variable pointer

ExpDate = StringByKey("state.internal.triggerTimeString", Headerwave[106], "=")
ExpDate = ReplaceString("'", ExpDate, "")

if(stringmatch(ExpDate[1], "/"))
	Month = "0"+ExpDate[0]
	pointer = 1
Elseif(stringmatch(ExpDate[2], "/"))
	Month = ExpDate[0,1]
	pointer = 2
else
	DoAlert 0, "Something's wrong with the date..."
	return ""
endif

if(stringmatch(ExpDate[pointer+2], "/"))
	Day = "0"+ExpDate[pointer+1]
	pointer += 1
Elseif(stringmatch(ExpDate[pointer+3], "/"))
	Day = ExpDate[pointer+1,pointer+2]
	pointer += 2
else
	DoAlert 0, "Something's wrong with the day..."
	return ""
endif

year = ExpDate[pointer+2, pointer+5]

ExpTime=ExpDate[pointer+7, pointer+14]

string result = Day+"/"+Month+"/"+Year+" "+ExpTime

Return Result
End

//////////////////////////////////////////////////////

Function ZSlicesFromHeader(HeaderWave)
wave /t HeaderWave

Return NumberByKey("state.acq.numberOfZSlices", Headerwave[3], "=")

End

//////////////////////////////////////////////////////

Function ZStepsizeFromHeader(HeaderWave)
wave /t HeaderWave

Return NumberByKey("state.acq.zStepSize", Headerwave[4], "=")

End

//////////////////////////////////////////////////////

Function GetHeaderInfoFromWave(Wave3D)
Wave Wave3D
string IACPHeader_Wavename, StringHeader_Wavename, SIHeaderWave

variable zoom = 1, x_res, y_res, z_res, x_dim, y_dim, z_dim, timeperline = 1
string expdate, age, construct, filename , settings, notes

IACPHeader_Wavename=nameofwave(wave3d)+"_head"
StringHeader_Wavename=nameofwave(wave3d)+"_string"
SIHeaderWave = nameofwave(wave3d)+"_header"

if (!(waveexists($SIHeaderWave)))
	prompt SIHeaderWave, "Specify the wave containing the ScanImage header info:", popup,WaveList("*",";","Text:1")
	DoPrompt "File countaining ScanImage header", SIheaderWave
		if (V_flag)
			return -1
		endif
endif



if (!(waveexists($IACPHeader_Wavename)))
	make /o/n=5 headcalcwave	
else
	duplicate /o $IACPHeader_Wavename, headcalcwave
	zoom = headcalcwave[0]
	x_res = headcalcwave[1]
	y_res= headcalcwave[2]
	z_res =  headcalcwave[3]
	timeperline = headcalcwave[4]
endif


if (!(waveexists($StringHeader_Wavename)))
	make /o/t/n= 7 stringheadwave
	filename = nameofwave(wave3d)
	expdate = date()
else
	duplicate /o /t $StringHeader_Wavename, stringheadwave
	expdate = stringheadwave[0]
	age = stringheadwave[1]
	construct = stringheadwave[2]
	filename = stringheadwave[3]
	settings = stringheadwave[4]
	notes = stringheadwave[6]
endif

wave /t stringheadwave

Zoom=ZoomFromHeader($SIHeaderWave)
timeperline=msPerLineFromHeader($SIHeaderWave)
expdate=ExpDateFromHeader($SIHeaderWave)
//prompt age, "Age of test subject"
//prompt construct, "Construct"
//prompt filename, "Filename of stack"
//prompt settings, "Microscope settings"
//prompt notes, "Any other notes"



//sidelength = 665.6 m / zoom
x_res = 665.6 / zoom / dimsize(wave3d,0)	//in m
y_res = 665.6 / zoom / dimsize(wave3d,1)	//in m
z_res = timeperline *  dimsize(wave3d,1)	//in ms

 headcalcwave[0] = zoom
 headcalcwave[1] = x_res
 headcalcwave[2] = y_res
 headcalcwave[3] = z_res 
 headcalcwave[4] = timeperline
 
 
 stringheadwave[0] =  expdate
 //stringheadwave[1] = age
 //stringheadwave[2] = construct
 //stringheadwave[3] = filename 
 //stringheadwave[4] = settings 
 //stringheadwave[5] = ""
 //stringheadwave[6] = notes
 




duplicate /o headcalcwave , $IACPHeader_Wavename		
duplicate /o /t stringheadwave, $StringHeader_Wavename	
killwaves/z headcalcwave, stringheadwave				
return 1													
end
