//m-sequences
#pragma version = 1.0
//written in Igor Pro v.6.02A
//by M. Manu Dorostkar
//October, 2007

//Syntax:
//mls*n*(stop, resultname)
//n...length of shift register, integer (1-20); use without asterisks, e.g. mls7(127,"filename")
//Period length = 2 ^ n - 1
//<stop>...number of values generated (rounded by program to positive integer)
//<resultname>...name of the wave

//		//<--- indicates lines that must be adapted, if register length is changed

//just for reference
//$m$ 	Recursion relation
//1 	$a_{i+1} = a_{i}$
//2 	$a_{i+2} = a_{i+1} + a_i$
//3 	$a_{i+3} = a_{i+1} + a_i$
//4 	$a_{i+4} = a_{i+1} + a_i$
//5 	$a_{i+5} = a_{i+2} + a_i$
//6 	$a_{i+6} = a_{i+1} + a_i$
//7 	$a_{i+7} = a_{i+1} + a_i$
//8 	$a_{i+8} = a_{i+6} + a_{i+5} + a_{i+1} + a_i$
//9 	$a_{i+9} = a_{i+4} + a_i$
//10 	$a_{i+10} = a_{i+3} + a_i$
//11 	$a_{i+11} = a_{i+2} + a_i$
//12 	$a_{i+12} = a_{i+7} + a_{i+4} + a_{i+3} + a_i$
//13 	$a_{i+13} = a_{i+4} + a_{i+3} + a_{i+1} + a_i$
//14 	$a_{i+14} = a_{i+12} + a_{i+11} + a_{i+1} + a_i$
//15 	$a_{i+15} = a_{i+1} + a_i$
//16 	$a_{i+16} = a_{i+5} + a_{i+3} + a_{i+2} + a_i$
//17 	$a_{i+17} = a_{i+3} + a_i$
//18 	$a_{i+18} = a_{i+7} + a_i$
//19 	$a_{i+19} = a_{i+6} + a_{i+5} + a_{i+1} + a_i$
//20 	$a_{i+20} = a_{i+3} + a_i$
// taken from http://www.ph.ed.ac.uk/~jonathan/thesis/node83.html

///////////////////////////////////////////////////////////////////////////
//////////////////////////////Code starts below///////////////////////////////
///////////////////////////////////////////////////////////////////////////

#pragma rtGlobals=1		// Use modern global access method.

function mls3(stop, resultname) 

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 3											//n=length of shift register (mls*n*)

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[1]%^mlswave1[0]) 	 //set new bit
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////



function mls4(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 4											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[1]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////



function mls5(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 5											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[2]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////


function mls6(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 6											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[1]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////


function mls7(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 7											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[1]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////


function mls8(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 8											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[6]%^mlswave1[5]%^mlswave1[1]%^mlswave1[0]) 	 //set new bit		//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls9(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 9											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[4]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls10(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 10											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[3]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls11(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 11											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[2]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls12(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 12											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[7]%^mlswave1[4]%^mlswave1[3]%^mlswave1[0]) 	 //set new bit	//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls13(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 13											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[4]%^mlswave1[3]%^mlswave1[1]%^mlswave1[0]) 	 //set new bit	//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls14(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 14											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[12]%^mlswave1[11]%^mlswave1[1]%^mlswave1[0]) 	 //set new bit	//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls15(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 15											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[1]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls16(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 16											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[5]%^mlswave1[3]%^mlswave1[2]%^mlswave1[0]) 	 //set new bit	//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls17(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 17											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[3]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls18(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 18											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[7]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls19(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 19											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[6]%^mlswave1[5]%^mlswave1[1]%^mlswave1[0]) 	 //set new bit	//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////

function mls20(stop, resultname) 																		//<---

variable stop
string resultname										
variable maxlength, mlsres = 0, counter = 0, count20, wavelength 
variable n = 20											//n=length of shift register (mls*n*)			//<---

maxlength = (2^n - 1)

Print "Maximum length sequence generator initiated."
Print "Maximum sequence length =",maxlength
Print "Generating", stop, "results with", (floor(stop/maxlength)),"repetitions..."


make/o/n=(n) mlswave1 = 1			//registers
make/o/n=(n) mlswave2 = 0


wavelength = round(abs(stop))
make/o/n=(wavelength) mlsresult = 0
 

do
	mlswave2=0
	count20 = 0
	mlswave2[n] = (mlswave1[3]%^mlswave1[0]) 	 //set new bit									//<---
	//bit operator depending on max. number taken from a table, out of a book
	mlsresult[counter] = mlswave2[20] 				//new bit = output 

	do
		mlswave2[(count20)] = mlswave1[(count20 + 1)] //shift rest of wave
		count20 = count20 + 1
	while (count20 < (n-1))
	 
	 mlswave1=mlswave2								//swap waves
	counter = counter + 1
while (counter < wavelength)

make/o/n=(wavelength) $resultname = mlsresult
killwaves/z mlswave1, mlswave2, mlsresult
print "Saved results to a wave named >>"+nameofwave($resultname)+"<<."
end


///////////////////////////////////////////////////////////////////////////